from enum import IntEnum


class VoipTerminationType(IntEnum):
    UNDEFINED = 0
    NOT_ALLOCATED_NUMBER = 1
    NOT_FOUND_USER = 11
    NORMAL_RELEASE = 16
    USER_BUSY = 17
    NOT_USER_RESPONDING = 18
    RESOURCE_UNAVAILABLE = 47
    CALL_REJECTED = 21
    CALL_CANCELED = 77
    CALL_HAVING_REQUEST_CALL_ID_HAS_BEEN = 86
    MAX = 127



from enum import IntEnum


class ChannelType(IntEnum):
    UNKNOWN = 0
    PRIVATE = 1
    ROOM = 2
    TEAM = 3
    GROUP = 4
    BOT = 5
    EXTERNAL_PRIVATE = 6
    EXTERNAL_ROOM = 7
    EXTERNAL_GROUP = 9
    EXTERNAL_LINE_GROUP = 10
    PTT = 12



from enum import Enum


class MessageType(Enum):
    CONNECTION_REQUEST = "CONNECTION_REQUEST"
    CONNECTION_RESPONSE = "CONNECTION_RESPONSE"
    ALIVE_REQUEST = "ALIVE_REQUEST"
    ALIVE_RESPONSE = "ALIVE_RESPONSE"
    NEW_NOTIFICATION = "NEW_NOTIFICATION"
    PRESENCE = "PRESENCE"



import uuid

from line_works.mqtt.config import KEEP_ALIVE_INTERVAL_SEC
from line_works.mqtt.enums.packet_type import PacketType


class ConnectionPacket(bytearray):
    def __append_utf8_bytes(self, b: bytes) -> None:
        self.extend(len(b).to_bytes(2, byteorder="big"))
        self.extend(b)

    def __append_remaining_length(self, length: int) -> None:
        while True:
            byte = length % 128
            length //= 128
            if length > 0:
                byte |= 0x80  # Set the continuation bit
            self.append(byte)
            if length == 0:
                break

    def generate(self) -> bytes:
        client_id = str(uuid.uuid4()).replace("-", "")[:22]
        client_id_bytes = client_id.encode("utf-8")
        username_bytes = "dummy".encode("utf-8")
        password_bytes = client_id_bytes
        protocol_name_bytes = "MQTT".encode("utf-8")

        remaining_length = (
            2  # Protocol name
            + len(protocol_name_bytes)
            + 1  # Protocol level
            + 1  # Connect flags
            + 2  # Keep alive
            + 2  # Client ID
            + len(client_id_bytes)
            + 2  # User name
            + len(username_bytes)
            + 2  # Password
            + len(password_bytes)
        )

        packet_type = PacketType.CONNECT.value << 4
        self.append(packet_type)
        self.__append_remaining_length(remaining_length)

        self.__append_utf8_bytes(protocol_name_bytes)
        self.append(0x04)  # Protocol level: MQTT v3.1.1
        self.append(0xC6)  # Connect flags: 11000110
        self.extend(KEEP_ALIVE_INTERVAL_SEC.to_bytes(2, byteorder="big"))

        self.__append_utf8_bytes(client_id_bytes)
        self.__append_utf8_bytes(username_bytes)
        self.__append_utf8_bytes(password_bytes)

        return bytes(self)



from enum import IntEnum


class MessageType(IntEnum):
    UNKNOWN = 0
    TEXT = 1
    VOIP = 2
    CONTACT = 3
    LOCATION = 4
    BOT = 5
    OFFICAL = 6
    NOTI = 7
    RICH = 8
    NEW_RICH = 10
    IMAGE = 11
    AUDIO = 12
    RECORD = 13
    VIDEO = 14
    STICKER_OLD = 15
    FILE = 16
    TEAM_NOTE = 17
    STICKER_V3 = 18
    NOTE_COMMENT = 19
    VOIP_GROUP_CALL_SCREEN_SHARE = 21
    MERGE_FORWARD = 22
    BOT_LINK_IMAGE = 20
    BOT_BUTTONS = 23
    BOT_LIST = 24
    BOT_LINK_RICH = 25
    PROFILE = 26
    BOT_RICH = 27
    NOTE_WITH_MENTION = 28
    TEMPLATE = 29
    LINE_GROUP_AUTO = 30
    REPLY_RICH = 37
    REPLY_STICKER = 38
    VOICE = 39
    BOT_FILE = 44
    BOT_MENTION = 46
    BOT_LINK_MENTION = 47
    BOT_RICH_BUTTON_MENTION = 48
    EXTERNAL_TEMPLATE = 49
    PTTSTT = 50
    FOCUS_BOT_FLEX = 95
    BOT_FLEX = 96
    BOT_CAROUSEL = 97
    BOT_IMAGE_CAROUSEL = 98
    VOIP2 = 100
    SYS_JOIN = 101
    SYS_QUIT = 102
    SYS_TITLE_CHANGE = 103
    KICK_BOT = 104
    CHAT_ROOM_PICTURE_CHANGE = 105
    GROUP_TEAM_NOTE = 106
    OUT_BLOCK = 108
    SWITCH_GROUP_ROOM = 109
    SCREEN_SHARE_INVITE = 107
    SCREEN_SHARE_STOP = 107
    SYS_SCREEN_SHARE = 110
    TRANSLATE_SYSTEM = 111
    FOLDER_SET_SYSTEM = 112
    CALENDAR_USE = 113
    DRIVE_USE = 114
    HISTORY_MODE = 115
    SYS_BIG_ROOM_CHAT = 116
    LINE_GROUP_KICKED_USER = 117
    TRANSLATE_SYSTEM_V2 = 118
    TASK_SYSTEM = 119
    SYS_SETTING = 120

    # NOTE: 現在V3が使われているようなので、18をNOTIFICATION_STICKERとして定義
    STICKER = STICKER_V3



from enum import StrEnum


class NotificationSubType(StrEnum):
    # INVITE_EXTERNAL_FRIEND
    MENTION = "REV_MENTION_MSG"
    NOTE_COMMENT_MENTION = "REV_MENTION_NOTE_COMMENT"
    INVITE_LINE = "REV_ADD_FRIEND_LINE"
    INVITE_WORKSAT = "REV_ADD_FRIEND_WORKSAT"
    # NOTE_POST
    NOTE_WRITE = "REV_NOTE_WRITE"
    NOTE_NOTICE = "REV_NOTE_NOTICE"
    NOTE_SHARE = "REV_NOTE_SHARE"
    # GROUP_CALL_AND_SCREEN_SHARE
    GROUP_VOICE_CALL_INVITE = "REV_GROUP_CALL_INVITE"
    GROUP_VOICE_CALL_START = "REV_GROUP_CALL_START"
    GROUP_SCREEN_SHARE_INVITE = "REV_GROUP_RD_INVITE"
    GROUP_SCREEN_SHARE_START = "REV_GROUP_RD_START"
    GROUP_VIDEO_CALL_INVITE = "REV_GROUP_VCALL_INVITE"
    GROUP_VIDEO_CALL_START = "REV_GROUP_VCALL_START"
    GROUP_VOICE_CALL_AND_SCREEN_SHARE_INVITE = "REV_GROUP_CALL_RD_INVITE"
    GROUP_VIDEO_CALL_AND_SCREEN_SHARE_INVITE = "REV_GROUP_VCALL_RD_INVITE"



from enum import IntEnum


class NotificationType(IntEnum):
    NORMAL = 101
    AWAY = 102
    LEAVE = 202
    INVITE = 203
    KICK = 204

    # 通知メッセージタイプ
    NOTIFICATION_UNKNOWN = 0
    NOTIFICATION_MESSAGE = 1
    NOTIFICATION_CONTACT = 3
    NOTIFICATION_LOCATION = 4
    NOTIFICATION_BOT = 5
    NOTIFICATION_OFFICIAL = 6
    NOTIFICATION_CHANNELNOTI = 7
    NOTIFICATION_RICH = 8
    NOTIFICATION_NEW_RICH = 10
    NOTIFICATION_IMAGE = 11
    NOTIFICATION_AUDIO = 12
    NOTIFICATION_RECORD = 13
    NOTIFICATION_VIDEO = 14
    NOTIFICATION_STICKER_OLD = 15
    NOTIFICATION_STICKER_V3 = 18
    NOTIFICATION_FILE = 16
    NOTIFICATION_LOCATION_EMOJI = 27
    NOTIFICATION_CALL = 31
    NOTIFICATION_VCALL = 32
    NOTIFICATION_INVITE_DSSHARE = 34
    NOTIFICATION_STOP_DSSHARE = 35
    NOTIFICATION_CANCEL_CALL = 33
    NOTIFICATION_NEW_MAIL = 40
    NOTIFICATION_BADGE = 41
    NOTIFICATION_CAL_SCHD_REMIND = 50
    NOTIFICATION_CAL_TASK_REMIND = 51
    NOTIFICATION_CAL_APPO_INVITE = 52
    NOTIFICATION_CAL_APPO_UPDATE = 53
    NOTIFICATION_CAL_APPO_CANCEL = 54
    NOTIFICATION_API_EXEC_COMMAND = 61
    NOTIFICATION_SERVICE = 100

    # NOTE: 現在V3が使われているようなので、18をNOTIFICATION_STICKERとして定義
    NOTIFICATION_STICKER = NOTIFICATION_STICKER_V3



from enum import IntFlag


class ServiceType(IntFlag):
    UNKNOWN = 0x0000
    VOIP = 0x0001
    MESSAGE = 0x0002
    MAIL = 0x0004
    CALENDAR = 0x0008
    API = 0x0010
    BOARD = 0x0040
    DRIVE = 0x0080
    SCREEN_SHARE = 0x0100
    ASSISTANT = 0x0200
    TASK = 0x0400
    SERVICE = 0x0800



from enum import IntEnum


class ServiceSubType(IntEnum):
    ADMIN = 1
    BOARD = 2
    CALENDAR = 3
    CONTACT = 4
    TASK = 5
    FORM = 6
    DRIVE = 7
    APPROVAL = 8
    RESERVATION = 9
    MYWORK = 10
    CLOVANOTE = 11
    CSTALK = 12



from enum import IntEnum


class PacketType(IntEnum):
    CONNECT = 1  # 接続リクエスト
    CONNACK = 2  # 接続応答
    PUBLISH = 3  # メッセージ配信
    PUBACK = 4  # QoS 1 配信確認
    PUBREC = 5  # QoS 2 受信確認 (Part 1)
    PUBREL = 6  # QoS 2 配信リリース (Part 2)
    PUBCOMP = 7  # QoS 2 配信完了 (Part 3)
    SUBSCRIBE = 8  # 購読リクエスト
    SUBACK = 9  # 購読応答
    UNSUBSCRIBE = 10  # 購読解除リクエスト
    UNSUBACK = 11  # 購読解除応答
    PINGREQ = 12  # 接続維持リクエスト
    PINGRESP = 13  # 接続維持応答
    DISCONNECT = 14  # 接続終了通知



from enum import IntEnum


class ControlPacketType(IntEnum):
    NORMAL = 93001
    ACK = 93004
    RECALL = 93005
    JOIN = 94001
    QUIT = 94002
    SYSTEM = 94003
    CHANNELSYNC = 94004
    KICK = 94005
    BOTMENU = 95001
    LARGE = 93006
    REACTION = 93007
    VOIPJOINCOUNT = 96005
    PTT_MIC_ON = 96007
    PTT_MIC_OFF = 96008
